/* ide-project-edit-private.h
 *
 * Copyright (C) 2016 Christian Hergert <chergert@redhat.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef IDE_PROJECT_EDIT_PRIVATE_H
#define IDE_PROJECT_EDIT_PRIVATE_H

#include "buffers/ide-buffer.h"
#include "projects/ide-project-edit.h"

G_BEGIN_DECLS

void _ide_project_edit_prepare (IdeProjectEdit *self,
                                IdeBuffer      *buffer);
void _ide_project_edit_apply   (IdeProjectEdit *self,
                                IdeBuffer      *buffer);

G_END_DECLS

#endif /* IDE_PROJECT_EDIT_PRIVATE_H */
